// PPCToolTips.cpp: fichier d'implmentation
//

#include "stdafx.h"
#include "PPCToolTips.h"


// CPPCToolTips
BEGIN_MESSAGE_MAP(CPPCToolTips, CWnd)
	ON_WM_PAINT()
	ON_WM_TIMER()
END_MESSAGE_MAP()

IMPLEMENT_DYNAMIC(CPPCToolTips, CWnd)

CPPCToolTips::CPPCToolTips()
{
	WNDCLASS Class;
	memset(&Class, 0, sizeof(Class));
	Class.style = WS_BORDER | WS_POPUP;
	Class.hInstance = AfxGetInstanceHandle();
	Class.hbrBackground = (HBRUSH)COLOR_BACKGROUND;
	Class.lpfnWndProc = ::DefWindowProc;
	Class.lpszClassName = PPC_CLASS_NAME;

	AfxRegisterClass(&Class);

	m_hWnd = NULL;
	m_nTimerFunc = TIMERF_NONE;

	m_clrBkColor = DEFBKCOLOR;
	m_clrTextColor = DEFTXTCOLOR;
	m_nTimeShow = DEFTIME_SHOW;
	m_nTimeErase = DEFTIME_ERASE;
	m_nTimeMax = DEFTIME_MAX;
}

CPPCToolTips::~CPPCToolTips()
{
	if(m_hWnd)
		Desactivate();
}

// Activate ToolTips and create window
BOOL CPPCToolTips::Activate(CPoint point, CWnd* pParent)
{
	if(!m_szTipText.IsEmpty() && !m_hWnd)
	{
		CSize TTSize = pParent->GetDC()->GetTextExtent(m_szTipText);
		TTSize.cx += TT_MARGE;
		TTSize.cy += TT_MARGE;

		CRect rcPos;
		rcPos.left = point.x - (TTSize.cx>>1);
		rcPos.right = rcPos.left + TTSize.cx;
		rcPos.bottom = point.y;
		rcPos.top = rcPos.bottom - TTSize.cy;

		pParent->ClientToScreen(rcPos);
		
		if(rcPos.right > GetSystemMetrics(SM_CXSCREEN))
		{
			rcPos.right = GetSystemMetrics(SM_CXSCREEN);
			rcPos.left = rcPos.right - TTSize.cx;
		}
		if(rcPos.left < 0)
		{
			rcPos.left = 0;
			rcPos.right = rcPos.left + TTSize.cx;
		}
		if(rcPos.bottom > GetSystemMetrics(SM_CYSCREEN))
		{
			rcPos.bottom = GetSystemMetrics(SM_CYSCREEN);
			rcPos.top = rcPos.bottom - TTSize.cy;
		}
		if(rcPos.top < 0)
		{
			rcPos.top = 0;
			rcPos.bottom = rcPos.top + TTSize.cy;
		}

		if(!CreateEx(WS_EX_TOPMOST, PPC_CLASS_NAME, NULL, WS_BORDER | WS_POPUP, rcPos, NULL, NULL))
			return FALSE;

		m_nTimerFunc = TIMERF_CREATE;
		SetTimer(IDTIMER, m_nTimeShow, NULL);

		return TRUE;
	}
	return FALSE;
}

// Destroy the tooltip window
void CPPCToolTips::Desactivate()
{
	if(m_hWnd)
	{
		if(m_nTimerFunc != TIMERF_NONE)
		{
			KillTimer(IDTIMER);
			m_nTimerFunc = TIMERF_NONE;
		}
		ShowWindow(SW_HIDE);
		InvalidateRect(NULL);
		DestroyWindow();
	}
	m_hWnd = NULL;
}

// Set the text of the tooltip
void CPPCToolTips::SetToolText(LPCTSTR szText)
{
	m_szTipText = szText;
}

// Message processing. The parent window has only to redirect messages, and the class do the job alone
void CPPCToolTips::ProcessMessage(MSG* pMsg, CWnd* pParent)
{
	switch(pMsg->message)
	{
		case WM_LBUTTONDOWN:
			{
				CPoint point;
				point.x = GET_X_LPARAM(pMsg->lParam);
				point.y = GET_Y_LPARAM(pMsg->lParam);
				Activate(point, pParent);
			}
			break;
		case WM_LBUTTONUP:
		case WM_RBUTTONUP:
		case WM_MOUSEMOVE:
			Desactivate();
			break;
		case WM_CONTEXTMENU:
			m_nTimerFunc = TIMERF_DEL;
			if(m_hWnd)
				SetTimer(IDTIMER, m_nTimeErase, NULL); 
			break;
	}
}

// Drawing of the ToolTip
void CPPCToolTips::OnPaint()
{
	CPaintDC dc(this); // device context for painting
	RECT rc;
	GetClientRect(&rc);
	dc.SetBkColor(m_clrBkColor);
	dc.SetTextColor(m_clrTextColor);
	dc.FillSolidRect(&rc, m_clrBkColor);
	dc.DrawText(m_szTipText, &rc, DT_CENTER | DT_NOCLIP | DT_NOPREFIX);
}

// Timer processing
void CPPCToolTips::OnTimer(UINT_PTR nIDEvent)
{
	// Suppression systmatique du timer
	KillTimer(nIDEvent);

	switch(m_nTimerFunc)
	{
		case TIMERF_NONE:  // do nothing
			break;
		case TIMERF_CREATE: // Creation, show window then set the timer for max time
			ShowWindow(SW_SHOWNA);
			m_nTimerFunc = TIMERF_DEL;
			SetTimer(nIDEvent, m_nTimeMax, NULL);
			break;
		case TIMERF_DEL: // Hide window and destroy it
			Desactivate();
			m_nTimerFunc = TIMERF_NONE;
			break;
	}

	CWnd::OnTimer(nIDEvent);
}

// Sets the background color
void CPPCToolTips::SetBkColor(COLORREF clrColor)
{
	m_clrBkColor = clrColor;
	if(m_hWnd)
		InvalidateRect(NULL);
}

// Sets the text color
void CPPCToolTips::SetTextColor(COLORREF clrColor)
{
	m_clrTextColor = clrColor;
	if(m_hWnd)
		InvalidateRect(NULL);
}

// Sets the timers. Use -1 to not modify a timer
void CPPCToolTips::SetDelays(int nShow, int nErase, int nMax)
{
	if(nShow != -1)
		m_nTimeShow = nShow;
	if(nErase != -1)
		m_nTimeErase = nErase;
	if(nMax != -1)
		m_nTimeMax = nMax;
}
